@@ -1,7 +1,9 @@
-# Copyright 2008-2016 by Peter Cock.  All rights reserved.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
+# Copyright 2008-2018 by Peter Cock.  All rights reserved.
+#
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 
 """Multiple sequence alignment input/output as alignment objects.
 
@@ -61,16 +63,16 @@ Use the function Bio.AlignIO.write(...), which takes a complete set of
 Alignment objects (either as a list, or an iterator), an output file handle
 (or filename in recent versions of Biopython) and of course the file format::
 
-  from Bio import AlignIO
-  alignments = ...
-  count = SeqIO.write(alignments, "example.faa", "fasta")
+    from Bio import AlignIO
+    alignments = ...
+    count = SeqIO.write(alignments, "example.faa", "fasta")
 
 If using a handle make sure to close it to flush the data to the disk::
 
-  from Bio import AlignIO
-  alignments = ...
-  with open("example.faa", "w") as handle:
-    count = SeqIO.write(alignments, handle, "fasta")
+    from Bio import AlignIO
+    alignments = ...
+    with open("example.faa", "w") as handle:
+        count = SeqIO.write(alignments, handle, "fasta")
 
 In general, you are expected to call this function once (with all your
 alignments) and then close the file handle.  However, for file formats
@@ -108,12 +110,14 @@ names are also used in Bio.SeqIO and include the following:
     readable output.
   - ig        - The IntelliGenetics file format, apparently the same as the
     MASE alignment format.
+  - msf       - The GCG MSF alignment format, originally from PileUp tool.
   - nexus     - Output from NEXUS, see also the module Bio.Nexus which can also
     read any phylogenetic trees in these files.
   - phylip    - Interlaced PHYLIP, as used by the PHLIP tools.
   - phylip-sequential - Sequential PHYLIP.
   - phylip-relaxed - PHYLIP like format allowing longer names.
   - stockholm - A richly annotated alignment file format used by PFAM.
+  - mauve - Output from progressiveMauve/Mauve
 
 Note that while Bio.AlignIO can read all the above file formats, it cannot
 write to all of them.
@@ -124,9 +128,6 @@ same length.
 """
 
 
-from __future__ import print_function
-from Bio._py3k import basestring
-
 # TODO
 # - define policy on reading aligned sequences with gaps in
 #   (e.g. - and . characters) including how the alphabet interacts
@@ -142,7 +143,6 @@ from Bio._py3k import basestring
 #   http://www.bioperl.org/wiki/MSF_multiple_alignment_format
 
 from Bio.Align import MultipleSeqAlignment
-from Bio.Align.Generic import Alignment
 from Bio.Alphabet import Alphabet, AlphabetEncoder, _get_base_alphabet
 from Bio.File import as_handle
 
@@ -152,42 +152,48 @@ from . import NexusIO
 from . import PhylipIO
 from . import EmbossIO
 from . import FastaIO
+from . import MafIO
+from . import MauveIO
+from . import MsfIO
 
 # Convention for format names is "mainname-subtype" in lower case.
 # Please use the same names as BioPerl and EMBOSS where possible.
 
 _FormatToIterator = {  # "fasta" is done via Bio.SeqIO
-                     "clustal": ClustalIO.ClustalIterator,
-                     "emboss": EmbossIO.EmbossIterator,
-                     "fasta-m10": FastaIO.FastaM10Iterator,
-                     "nexus": NexusIO.NexusIterator,
-                     "phylip": PhylipIO.PhylipIterator,
-                     "phylip-sequential": PhylipIO.SequentialPhylipIterator,
-                     "phylip-relaxed": PhylipIO.RelaxedPhylipIterator,
-                     "stockholm": StockholmIO.StockholmIterator,
-                     }
+    "clustal": ClustalIO.ClustalIterator,
+    "emboss": EmbossIO.EmbossIterator,
+    "fasta-m10": FastaIO.FastaM10Iterator,
+    "maf": MafIO.MafIterator,
+    "mauve": MauveIO.MauveIterator,
+    "msf": MsfIO.MsfIterator,
+    "nexus": NexusIO.NexusIterator,
+    "phylip": PhylipIO.PhylipIterator,
+    "phylip-sequential": PhylipIO.SequentialPhylipIterator,
+    "phylip-relaxed": PhylipIO.RelaxedPhylipIterator,
+    "stockholm": StockholmIO.StockholmIterator,
+}
 
 _FormatToWriter = {  # "fasta" is done via Bio.SeqIO
-                     # "emboss" : EmbossIO.EmbossWriter, (unfinished)
-                   "nexus": NexusIO.NexusWriter,
-                   "phylip": PhylipIO.PhylipWriter,
-                   "phylip-sequential": PhylipIO.SequentialPhylipWriter,
-                   "phylip-relaxed": PhylipIO.RelaxedPhylipWriter,
-                   "stockholm": StockholmIO.StockholmWriter,
-                   "clustal": ClustalIO.ClustalWriter,
-                   }
+    "clustal": ClustalIO.ClustalWriter,
+    "maf": MafIO.MafWriter,
+    "mauve": MauveIO.MauveWriter,
+    "nexus": NexusIO.NexusWriter,
+    "phylip": PhylipIO.PhylipWriter,
+    "phylip-sequential": PhylipIO.SequentialPhylipWriter,
+    "phylip-relaxed": PhylipIO.RelaxedPhylipWriter,
+    "stockholm": StockholmIO.StockholmWriter,
+}
 
 
 def write(alignments, handle, format):
     """Write complete set of alignments to a file.
 
     Arguments:
-      - alignments - A list (or iterator) of Alignment objects (ideally the
-        new MultipleSeqAlignment objects), or (if using Biopython
-        1.54 or later) a single alignment object.
-      - handle    - File handle object to write to, or filename as string
-        (note older versions of Biopython only took a handle).
-      - format    - lower case string describing the file format to write.
+     - alignments - A list (or iterator) of MultipleSeqAlignment objects,
+       or a single alignment object.
+     - handle    - File handle object to write to, or filename as string
+       (note older versions of Biopython only took a handle).
+     - format    - lower case string describing the file format to write.
 
     You should close the handle after calling this function.
 
@@ -196,18 +202,18 @@ def write(alignments, handle, format):
     from Bio import SeqIO
 
     # Try and give helpful error messages:
-    if not isinstance(format, basestring):
+    if not isinstance(format, str):
         raise TypeError("Need a string for the file format (lower case)")
     if not format:
         raise ValueError("Format required (lower case string)")
     if format != format.lower():
         raise ValueError("Format string '%s' should be lower case" % format)
 
-    if isinstance(alignments, Alignment):
+    if isinstance(alignments, MultipleSeqAlignment):
         # This raised an exception in older versions of Biopython
         alignments = [alignments]
 
-    with as_handle(handle, 'w') as fp:
+    with as_handle(handle, "w") as fp:
         # Map the file format to a writer class
         if format in _FormatToWriter:
             writer_class = _FormatToWriter[format]
@@ -217,42 +223,50 @@ def write(alignments, handle, format):
             # TODO - Can we make one call to SeqIO.write() and count the alignments?
             count = 0
             for alignment in alignments:
-                if not isinstance(alignment, Alignment):
-                    raise TypeError("Expect a list or iterator of Alignment "
-                                    "objects, got: %r" % alignment)
+                if not isinstance(alignment, MultipleSeqAlignment):
+                    raise TypeError(
+                        "Expect a list or iterator of MultipleSeqAlignment "
+                        "objects, got: %r" % alignment
+                    )
                 SeqIO.write(alignment, fp, format)
                 count += 1
         elif format in _FormatToIterator or format in SeqIO._FormatToIterator:
-            raise ValueError("Reading format '%s' is supported, but not writing"
-                             % format)
+            raise ValueError(
+                "Reading format '%s' is supported, but not writing" % format
+            )
         else:
             raise ValueError("Unknown format '%s'" % format)
 
-    assert isinstance(count, int), "Internal error - the underlying %s " \
-           "writer should have returned the alignment count, not %s" \
-           % (format, repr(count))
+    if not isinstance(count, int):
+        raise RuntimeError(
+            "Internal error - the underlying %s "
+            "writer should have returned the alignment count, not %s"
+            % (format, repr(count))
+        )
 
     return count
 
 
 # This is a generator function!
 def _SeqIO_to_alignment_iterator(handle, format, alphabet=None, seq_count=None):
-    """Uses Bio.SeqIO to create an MultipleSeqAlignment iterator (PRIVATE).
+    """Use Bio.SeqIO to create an MultipleSeqAlignment iterator (PRIVATE).
 
     Arguments:
-      - handle    - handle to the file.
-      - format    - string describing the file format.
-      - alphabet  - optional Alphabet object, useful when the sequence type
-        cannot be automatically inferred from the file itself
-        (e.g. fasta, phylip, clustal)
-      - seq_count - Optional integer, number of sequences expected in each
-        alignment.  Recommended for fasta format files.
+     - handle    - handle to the file.
+     - format    - string describing the file format.
+     - alphabet  - optional Alphabet object, useful when the sequence type
+       cannot be automatically inferred from the file itself
+       (e.g. fasta, phylip, clustal)
+     - seq_count - Optional integer, number of sequences expected in each
+       alignment.  Recommended for fasta format files.
 
     If count is omitted (default) then all the sequences in the file are
     combined into a single MultipleSeqAlignment.
     """
     from Bio import SeqIO
-    assert format in SeqIO._FormatToIterator
+
+    if format not in SeqIO._FormatToIterator:
+        raise ValueError("Unknown format '%s'" % format)
 
     if seq_count:
         # Use the count to split the records into batches.
@@ -279,17 +293,21 @@ def _force_alphabet(alignment_iterator, alphabet):
     # Assume the alphabet argument has been pre-validated
     given_base_class = _get_base_alphabet(alphabet).__class__
     for align in alignment_iterator:
-        if not isinstance(_get_base_alphabet(align._alphabet),
-                          given_base_class):
-            raise ValueError("Specified alphabet %s clashes with "
-                             "that determined from the file, %s"
-                             % (repr(alphabet), repr(align._alphabet)))
+        if not isinstance(_get_base_alphabet(align._alphabet), given_base_class):
+            raise ValueError(
+                "Specified alphabet %s clashes with "
+                "that determined from the file, %s"
+                % (repr(alphabet), repr(align._alphabet))
+            )
         for record in align:
-            if not isinstance(_get_base_alphabet(record.seq.alphabet),
-                              given_base_class):
-                raise ValueError("Specified alphabet %s clashes with "
-                                 "that determined from the file, %s"
-                           % (repr(alphabet), repr(record.seq.alphabet)))
+            if not isinstance(
+                _get_base_alphabet(record.seq.alphabet), given_base_class
+            ):
+                raise ValueError(
+                    "Specified alphabet %s clashes with "
+                    "that determined from the file, %s"
+                    % (repr(alphabet), repr(record.seq.alphabet))
+                )
             record.seq.alphabet = alphabet
         align._alphabet = alphabet
         yield align
@@ -299,14 +317,14 @@ def parse(handle, format, seq_count=None, alphabet=None):
     """Iterate over an alignment file as MultipleSeqAlignment objects.
 
     Arguments:
-      - handle    - handle to the file, or the filename as a string
-        (note older versions of Biopython only took a handle).
-      - format    - string describing the file format.
-      - alphabet  - optional Alphabet object, useful when the sequence type
-        cannot be automatically inferred from the file itself
-        (e.g. fasta, phylip, clustal)
-      - seq_count - Optional integer, number of sequences expected in each
-        alignment.  Recommended for fasta format files.
+     - handle    - handle to the file, or the filename as a string
+       (note older versions of Biopython only took a handle).
+     - format    - string describing the file format.
+     - alphabet  - optional Alphabet object, useful when the sequence type
+       cannot be automatically inferred from the file itself
+       (e.g. fasta, phylip, clustal)
+     - seq_count - Optional integer, number of sequences expected in each
+       alignment.  Recommended for fasta format files.
 
     If you have the file name in a string 'filename', use:
 
@@ -324,7 +342,7 @@ def parse(handle, format, seq_count=None, alphabet=None):
     If you have a string 'data' containing the file contents, use::
 
       from Bio import AlignIO
-      from StringIO import StringIO
+      from io import StringIO
       my_iterator = AlignIO.parse(StringIO(data), format)
 
     Use the Bio.AlignIO.read() function when you expect a single record only.
@@ -332,19 +350,20 @@ def parse(handle, format, seq_count=None, alphabet=None):
     from Bio import SeqIO
 
     # Try and give helpful error messages:
-    if not isinstance(format, basestring):
+    if not isinstance(format, str):
         raise TypeError("Need a string for the file format (lower case)")
     if not format:
         raise ValueError("Format required (lower case string)")
     if format != format.lower():
         raise ValueError("Format string '%s' should be lower case" % format)
-    if alphabet is not None and not (isinstance(alphabet, Alphabet) or
-                                     isinstance(alphabet, AlphabetEncoder)):
+    if alphabet is not None and not (
+        isinstance(alphabet, Alphabet) or isinstance(alphabet, AlphabetEncoder)
+    ):
         raise ValueError("Invalid alphabet, %s" % repr(alphabet))
     if seq_count is not None and not isinstance(seq_count, int):
         raise TypeError("Need integer for seq_count (sequences per alignment)")
 
-    with as_handle(handle, 'rU') as fp:
+    with as_handle(handle) as fp:
         # Map the file format to a sequence iterator:
         if format in _FormatToIterator:
             iterator_generator = _FormatToIterator[format]
@@ -356,34 +375,31 @@ def parse(handle, format, seq_count=None, alphabet=None):
                     i = iterator_generator(fp, seq_count, alphabet=alphabet)
                 except TypeError:
                     # It isn't supported.
-                    i = _force_alphabet(iterator_generator(fp, seq_count),
-                                        alphabet)
+                    i = _force_alphabet(iterator_generator(fp, seq_count), alphabet)
 
         elif format in SeqIO._FormatToIterator:
             # Exploit the existing SeqIO parser to the dirty work!
-            i = _SeqIO_to_alignment_iterator(fp, format,
-                                                alphabet=alphabet,
-                                                seq_count=seq_count)
+            i = _SeqIO_to_alignment_iterator(
+                fp, format, alphabet=alphabet, seq_count=seq_count
+            )
         else:
             raise ValueError("Unknown format '%s'" % format)
 
-        # This imposes some overhead... wait until we drop Python 2.4 to fix it
-        for a in i:
-            yield a
+        yield from i
 
 
 def read(handle, format, seq_count=None, alphabet=None):
-    """Turns an alignment file into a single MultipleSeqAlignment object.
+    """Turn an alignment file into a single MultipleSeqAlignment object.
 
     Arguments:
-      - handle    - handle to the file, or the filename as a string
-        (note older versions of Biopython only took a handle).
-      - format    - string describing the file format.
-      - alphabet  - optional Alphabet object, useful when the sequence type
-        cannot be automatically inferred from the file itself
-        (e.g. fasta, phylip, clustal)
-      - seq_count - Optional integer, number of sequences expected in each
-        alignment.  Recommended for fasta format files.
+     - handle    - handle to the file, or the filename as a string
+       (note older versions of Biopython only took a handle).
+     - format    - string describing the file format.
+     - alphabet  - optional Alphabet object, useful when the sequence type
+       cannot be automatically inferred from the file itself
+       (e.g. fasta, phylip, clustal)
+     - seq_count - Optional integer, number of sequences expected in each
+       alignment.  Recommended for fasta format files.
 
     If the handle contains no alignments, or more than one alignment,
     an exception is raised.  For example, using a PFAM/Stockholm file
@@ -421,30 +437,32 @@ def read(handle, format, seq_count=None, alphabet=None):
     """
     iterator = parse(handle, format, seq_count, alphabet)
     try:
-        first = next(iterator)
+        alignment = next(iterator)
     except StopIteration:
-        first = None
-    if first is None:
-        raise ValueError("No records found in handle")
+        raise ValueError("No records found in handle") from None
     try:
-        second = next(iterator)
-    except StopIteration:
-        second = None
-    if second is not None:
+        next(iterator)
         raise ValueError("More than one record found in handle")
+    except StopIteration:
+        pass
     if seq_count:
-        assert len(first) == seq_count
-    return first
+        if len(alignment) != seq_count:
+            raise RuntimeError(
+                "More sequences found in alignment than specified in seq_count: %s."
+                % seq_count
+            )
+    return alignment
 
 
 def convert(in_file, in_format, out_file, out_format, alphabet=None):
     """Convert between two alignment files, returns number of alignments.
 
-        - in_file - an input handle or filename
-        - in_format - input file format, lower case string
-        - output - an output handle or filename
-        - out_file - output file format, lower case string
-        - alphabet - optional alphabet to assume
+    Arguments:
+     - in_file - an input handle or filename
+     - in_format - input file format, lower case string
+     - output - an output handle or filename
+     - out_file - output file format, lower case string
+     - alphabet - optional alphabet to assume
 
     **NOTE** - If you provide an output filename, it will be opened which will
     overwrite any existing file without warning. This may happen if even the
@@ -452,13 +470,13 @@ def convert(in_file, in_format, out_file, out_format, alphabet=None):
     """
     # TODO - Add optimised versions of important conversions
     # For now just off load the work to SeqIO parse/write
-    with as_handle(in_file, 'rU') as in_handle:
+    with as_handle(in_file) as in_handle:
         # Don't open the output file until we've checked the input is OK:
         alignments = parse(in_handle, in_format, None, alphabet)
 
         # This will check the arguments and issue error messages,
         # after we have opened the file which is a shame.
-        with as_handle(out_file, 'w') as out_handle:
+        with as_handle(out_file, "w") as out_handle:
             count = write(alignments, out_handle, out_format)
 
     return count
@@ -466,4 +484,5 @@ def convert(in_file, in_format, out_file, out_format, alphabet=None):
 
 if __name__ == "__main__":
     from Bio._utils import run_doctest
+
     run_doctest()
