@@ -1,24 +1,20 @@
 #!/usr/bin/env python
-# Created: Wed May 29 08:07:18 2002
-# thomas@cbs.dtu.dk, Cecilia.Alsmark@ebc.uu.se
-# Copyright 2001 by Thomas Sicheritz-Ponten and Cecilia Alsmark.
+# Copyright 2002 by Thomas Sicheritz-Ponten and Cecilia Alsmark.
 # Revisions copyright 2014 by Markus Piotrowski.
 # Revisions copyright 2014-2016 by Peter Cock.
 # All rights reserved.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
-
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 """Miscellaneous functions for dealing with sequences."""
 
-from __future__ import print_function
 
 import re
 from math import pi, sin, cos
 
 from Bio.Seq import Seq, MutableSeq
 from Bio import Alphabet
-from Bio.Alphabet import IUPAC
 from Bio.Data import IUPACData
 
 
@@ -29,7 +25,7 @@ from Bio.Data import IUPACData
 
 
 def GC(seq):
-    """Calculates G+C content, returns the percentage (float between 0 and 100).
+    """Calculate G+C content, return percentage (as float between 0 and 100).
 
     Copes mixed case sequences, and with the ambiguous nucleotide S (G or C)
     when counting the G and C content.  The percentage is calculated against
@@ -41,7 +37,7 @@ def GC(seq):
 
     Note that this will return zero for an empty sequence.
     """
-    gc = sum(seq.count(x) for x in ['G', 'C', 'g', 'c', 'S', 's'])
+    gc = sum(seq.count(x) for x in ["G", "C", "g", "c", "S", "s"])
     try:
         return gc * 100.0 / len(seq)
     except ZeroDivisionError:
@@ -49,7 +45,7 @@ def GC(seq):
 
 
 def GC123(seq):
-    """Calculates total G+C content plus first, second and third positions.
+    """Calculate G+C content: total, for first, second and third positions.
 
     Returns a tuple of four floats (percentages between 0 and 100) for the
     entire sequence, and the three codon positions.  e.g.
@@ -62,15 +58,15 @@ def GC123(seq):
     nucleotides.
     """
     d = {}
-    for nt in ['A', 'T', 'G', 'C']:
+    for nt in ["A", "T", "G", "C"]:
         d[nt] = [0, 0, 0]
 
     for i in range(0, len(seq), 3):
-        codon = seq[i:i + 3]
+        codon = seq[i : i + 3]
         if len(codon) < 3:
-            codon += '  '
+            codon += "  "
         for pos in range(0, 3):
-            for nt in ['A', 'T', 'G', 'C']:
+            for nt in ["A", "T", "G", "C"]:
                 if codon[pos] == nt or codon[pos] == nt.lower():
                     d[nt][pos] += 1
     gc = {}
@@ -78,12 +74,12 @@ def GC123(seq):
     nall = 0
     for i in range(0, 3):
         try:
-            n = d['G'][i] + d['C'][i] + d['T'][i] + d['A'][i]
-            gc[i] = (d['G'][i] + d['C'][i]) * 100.0 / n
+            n = d["G"][i] + d["C"][i] + d["T"][i] + d["A"][i]
+            gc[i] = (d["G"][i] + d["C"][i]) * 100.0 / n
         except Exception:  # TODO - ValueError?
             gc[i] = 0
 
-        gcall = gcall + d['G'][i] + d['C'][i]
+        gcall = gcall + d["G"][i] + d["C"][i]
         nall = nall + n
 
     gcall = 100.0 * gcall / nall
@@ -91,38 +87,40 @@ def GC123(seq):
 
 
 def GC_skew(seq, window=100):
-    """Calculates GC skew (G-C)/(G+C) for multiple windows along the sequence.
+    """Calculate GC skew (G-C)/(G+C) for multiple windows along the sequence.
 
     Returns a list of ratios (floats), controlled by the length of the sequence
     and the size of the window.
 
+    Returns 0 for windows without any G/C by handling zero division errors.
+
     Does NOT look at any ambiguous nucleotides.
     """
     # 8/19/03: Iddo: added lowercase
     values = []
     for i in range(0, len(seq), window):
-        s = seq[i: i + window]
-        g = s.count('G') + s.count('g')
-        c = s.count('C') + s.count('c')
-        skew = (g - c) / float(g + c)
+        s = seq[i : i + window]
+        g = s.count("G") + s.count("g")
+        c = s.count("C") + s.count("c")
+        try:
+            skew = (g - c) / float(g + c)
+        except ZeroDivisionError:
+            skew = 0.0
         values.append(skew)
     return values
 
 
-def xGC_skew(seq, window=1000, zoom=100,
-                         r=300, px=100, py=100):
-    """Calculates and plots normal and accumulated GC skew (GRAPHICS !!!)."""
-    try:
-        import Tkinter as tkinter  # Python 2
-    except ImportError:
-        import tkinter  # Python 3
+def xGC_skew(seq, window=1000, zoom=100, r=300, px=100, py=100):
+    """Calculate and plot normal and accumulated GC skew (GRAPHICS !!!)."""
+    import tkinter
 
     yscroll = tkinter.Scrollbar(orient=tkinter.VERTICAL)
     xscroll = tkinter.Scrollbar(orient=tkinter.HORIZONTAL)
-    canvas = tkinter.Canvas(yscrollcommand=yscroll.set,
-                            xscrollcommand=xscroll.set, background='white')
+    canvas = tkinter.Canvas(
+        yscrollcommand=yscroll.set, xscrollcommand=xscroll.set, background="white"
+    )
     win = canvas.winfo_toplevel()
-    win.geometry('700x700')
+    win.geometry("700x700")
 
     yscroll.config(command=canvas.yview)
     xscroll.config(command=canvas.xview)
@@ -135,13 +133,13 @@ def xGC_skew(seq, window=1000, zoom=100,
     x1, x2, y1, y2 = X0 - r, X0 + r, Y0 - r, Y0 + r
 
     ty = Y0
-    canvas.create_text(X0, ty, text='%s...%s (%d nt)' % (seq[:7], seq[-7:], len(seq)))
+    canvas.create_text(X0, ty, text="%s...%s (%d nt)" % (seq[:7], seq[-7:], len(seq)))
     ty += 20
-    canvas.create_text(X0, ty, text='GC %3.2f%%' % (GC(seq)))
+    canvas.create_text(X0, ty, text="GC %3.2f%%" % (GC(seq)))
     ty += 20
-    canvas.create_text(X0, ty, text='GC Skew', fill='blue')
+    canvas.create_text(X0, ty, text="GC Skew", fill="blue")
     ty += 20
-    canvas.create_text(X0, ty, text='Accumulated GC Skew', fill='magenta')
+    canvas.create_text(X0, ty, text="Accumulated GC Skew", fill="magenta")
     ty += 20
     canvas.create_oval(x1, y1, x2, y2)
 
@@ -157,7 +155,7 @@ def xGC_skew(seq, window=1000, zoom=100,
         y1 = Y0 + r1 * cos(alpha)
         x2 = X0 + r2 * sin(alpha)
         y2 = Y0 + r2 * cos(alpha)
-        canvas.create_line(x1, y1, x2, y2, fill='blue')
+        canvas.create_line(x1, y1, x2, y2, fill="blue")
         # accumulated GC skew
         r1 = r - 50
         r2 = r1 - acc
@@ -165,7 +163,7 @@ def xGC_skew(seq, window=1000, zoom=100,
         y1 = Y0 + r1 * cos(alpha)
         x2 = X0 + r2 * sin(alpha)
         y2 = Y0 + r2 * cos(alpha)
-        canvas.create_line(x1, y1, x2, y2, fill='magenta')
+        canvas.create_line(x1, y1, x2, y2, fill="magenta")
 
         canvas.update()
         start += window
@@ -174,22 +172,21 @@ def xGC_skew(seq, window=1000, zoom=100,
 
 
 def nt_search(seq, subseq):
-    """Search for a DNA subseq in sequence.
+    """Search for a DNA subseq in sequence, return list of [subseq, positions].
 
-    use ambiguous values (like N = A or T or C or G, R = A or G etc.)
-    searches only on forward strand
+    Use ambiguous values (like N = A or T or C or G, R = A or G etc.),
+    searches only on forward strand.
     """
-    pattern = ''
+    pattern = ""
     for nt in subseq:
         value = IUPACData.ambiguous_dna_values[nt]
         if len(value) == 1:
             pattern += value
         else:
-            pattern += '[%s]' % value
+            pattern += "[%s]" % value
 
     pos = -1
     result = [pattern]
-    l = len(seq)
     while True:
         pos += 1
         s = seq[pos:]
@@ -200,27 +197,25 @@ def nt_search(seq, subseq):
         result.append(pos)
     return result
 
-# }}}
 
 ######################################
 # Protein
 ######################
-# {{{
 
 
-def seq3(seq, custom_map=None, undef_code='Xaa'):
-    """Turn a one letter code protein sequence into one with three letter codes.
+def seq3(seq, custom_map=None, undef_code="Xaa"):
+    """Convert protein sequence from one-letter to three-letter code.
 
     The single required input argument 'seq' should be a protein sequence using
-    single letter codes, either as a python string or as a Seq or MutableSeq
+    single letter codes, either as a Python string or as a Seq or MutableSeq
     object.
 
     This function returns the amino acid sequence as a string using the three
     letter amino acid codes. Output follows the IUPAC standard (including
     ambiguous characters B for "Asx", J for "Xle" and X for "Xaa", and also U
-    for "Sel" and O for "Pyl") plus "Ter" for a terminator given as an asterisk.
-    Any unknown character (including possible gap characters), is changed into
-    'Xaa' by default.
+    for "Sel" and O for "Pyl") plus "Ter" for a terminator given as an
+    asterisk. Any unknown character (including possible gap characters),
+    is changed into 'Xaa' by default.
 
     e.g.
 
@@ -248,21 +243,22 @@ def seq3(seq, custom_map=None, undef_code='Xaa'):
     This function was inspired by BioPerl's seq3.
     """
     if custom_map is None:
-        custom_map = {'*': 'Ter'}
+        custom_map = {"*": "Ter"}
     # not doing .update() on IUPACData dict with custom_map dict
     # to preserve its initial state (may be imported in other modules)
-    threecode = dict(list(IUPACData.protein_letters_1to3_extended.items()) +
-                     list(custom_map.items()))
+    threecode = dict(
+        list(IUPACData.protein_letters_1to3_extended.items()) + list(custom_map.items())
+    )
     # We use a default of 'Xaa' for undefined letters
     # Note this will map '-' to 'Xaa' which may be undesirable!
-    return ''.join(threecode.get(aa, undef_code) for aa in seq)
+    return "".join(threecode.get(aa, undef_code) for aa in seq)
 
 
-def seq1(seq, custom_map=None, undef_code='X'):
-    """Turns a three-letter code protein sequence into one with single letter codes.
+def seq1(seq, custom_map=None, undef_code="X"):
+    """Convert protein sequence from three-letter to one-letter code.
 
     The single required input argument 'seq' should be a protein sequence
-    using three-letter codes, either as a python string or as a Seq or
+    using three-letter codes, either as a Python string or as a Seq or
     MutableSeq object.
 
     This function returns the amino acid sequence as a string using the one
@@ -274,21 +270,21 @@ def seq1(seq, custom_map=None, undef_code='X'):
 
     e.g.
 
-    >>> from Bio.SeqUtils import seq3
+    >>> from Bio.SeqUtils import seq1
     >>> seq1("MetAlaIleValMetGlyArgTrpLysGlyAlaArgTer")
     'MAIVMGRWKGAR*'
 
     The input is case insensitive, e.g.
 
-    >>> from Bio.SeqUtils import seq3
+    >>> from Bio.SeqUtils import seq1
     >>> seq1("METalaIlEValMetGLYArgtRplysGlyAlaARGTer")
     'MAIVMGRWKGAR*'
 
     You can set a custom translation of the codon termination code using the
     dictionary "custom_map" argument (defaulting to {'Ter': '*'}), e.g.
 
-    >>> seq1("MetAlaIleValMetGlyArgTrpLysGlyAlaArg***", custom_map={"***": "*"})
-    'MAIVMGRWKGAR*'
+    >>> seq1("MetAlaIleValMetGlyArgTrpLysGlyAla***", custom_map={"***": "*"})
+    'MAIVMGRWKGA*'
 
     You can also set a custom translation for non-amino acid characters, such
     as '-', using the "undef_code" argument, e.g.
@@ -303,39 +299,37 @@ def seq1(seq, custom_map=None, undef_code='X'):
 
     """
     if custom_map is None:
-        custom_map = {'Ter': '*'}
+        custom_map = {"Ter": "*"}
     # reverse map of threecode
     # upper() on all keys to enable caps-insensitive input seq handling
-    onecode = dict((k.upper(), v) for k, v in
-                   IUPACData.protein_letters_3to1_extended.items())
+    onecode = {k.upper(): v for k, v in IUPACData.protein_letters_3to1_extended.items()}
     # add the given termination codon code and custom maps
-    onecode.update((k.upper(), v) for (k, v) in custom_map.items())
-    seqlist = [seq[3 * i:3 * (i + 1)] for i in range(len(seq) // 3)]
-    return ''.join(onecode.get(aa.upper(), undef_code) for aa in seqlist)
+    onecode.update((k.upper(), v) for k, v in custom_map.items())
+    seqlist = [seq[3 * i : 3 * (i + 1)] for i in range(len(seq) // 3)]
+    return "".join(onecode.get(aa.upper(), undef_code) for aa in seqlist)
 
 
-# }}}
-
 ######################################
 # Mixed ???
 ######################
-# {{{
 
 
-def molecular_weight(seq, seq_type=None, double_stranded=False, circular=False,
-                     monoisotopic=False):
-    """Calculates the molecular weight of a DNA, RNA or protein sequence.
+def molecular_weight(
+    seq, seq_type=None, double_stranded=False, circular=False, monoisotopic=False
+):
+    """Calculate the molecular mass of DNA, RNA or protein sequences as float.
 
     Only unambiguous letters are allowed. Nucleotide sequences are assumed to
     have a 5' phosphate.
 
-        - seq: String or Biopython sequence object.
-        - seq_type: The default (None) is to take the alphabet from the seq argument,
-          or assume DNA if the seq argument is a string. Override this with
-          a string 'DNA', 'RNA', or 'protein'.
-        - double_stranded: Calculate the mass for the double stranded molecule?
-        - circular: Is the molecule circular (has no ends)?
-        - monoisotopic: Use the monoisotopic mass tables?
+    Arguments:
+     - seq: String or Biopython sequence object.
+     - seq_type: The default (None) is to take the alphabet from the seq
+       argument, or assume DNA if the seq argument is a string. Override this
+       with a string 'DNA', 'RNA', or 'protein'.
+     - double_stranded: Calculate the mass for the double stranded molecule?
+     - circular: Is the molecule circular (has no ends)?
+     - monoisotopic: Use the monoisotopic mass tables?
 
     Note that for backwards compatibility, if the seq argument is a string,
     or Seq object with a generic alphabet, and no seq_type is specified
@@ -380,27 +374,29 @@ def molecular_weight(seq, seq_type=None, double_stranded=False, circular=False,
     # Rewritten by Markus Piotrowski, 2014
 
     # Find the alphabet type
-    tmp_type = ''
+    tmp_type = ""
     if isinstance(seq, (Seq, MutableSeq)):
         base_alphabet = Alphabet._get_base_alphabet(seq.alphabet)
         if isinstance(base_alphabet, Alphabet.DNAAlphabet):
-            tmp_type = 'DNA'
+            tmp_type = "DNA"
         elif isinstance(base_alphabet, Alphabet.RNAAlphabet):
-            tmp_type = 'RNA'
+            tmp_type = "RNA"
         elif isinstance(base_alphabet, Alphabet.ProteinAlphabet):
-            tmp_type = 'protein'
+            tmp_type = "protein"
         elif isinstance(base_alphabet, Alphabet.ThreeLetterProtein):
-            tmp_type = 'protein'
+            tmp_type = "protein"
             # Convert to one-letter sequence. Have to use a string for seq1
             seq = Seq(seq1(str(seq)), alphabet=Alphabet.ProteinAlphabet())
         elif not isinstance(base_alphabet, Alphabet.Alphabet):
-            raise TypeError("%s is not a valid alphabet for mass calculations"
-                             % base_alphabet)
+            raise TypeError(
+                "%s is not a valid alphabet for mass calculations" % base_alphabet
+            )
         else:
             tmp_type = "DNA"  # backward compatibity
         if seq_type and tmp_type and tmp_type != seq_type:
-            raise ValueError("seq_type=%r contradicts %s from seq alphabet"
-                             % (seq_type, tmp_type))
+            raise ValueError(
+                "seq_type=%r contradicts %s from seq alphabet" % (seq_type, tmp_type)
+            )
         seq_type = tmp_type
     elif isinstance(seq, str):
         if seq_type is None:
@@ -408,26 +404,25 @@ def molecular_weight(seq, seq_type=None, double_stranded=False, circular=False,
     else:
         raise TypeError("Expected a string or Seq object, not seq=%r" % seq)
 
-    seq = ''.join(str(seq).split()).upper()  # Do the minimum formatting
+    seq = "".join(str(seq).split()).upper()  # Do the minimum formatting
 
-    if seq_type == 'DNA':
+    if seq_type == "DNA":
         if monoisotopic:
             weight_table = IUPACData.monoisotopic_unambiguous_dna_weights
         else:
             weight_table = IUPACData.unambiguous_dna_weights
-    elif seq_type == 'RNA':
+    elif seq_type == "RNA":
         if monoisotopic:
             weight_table = IUPACData.monoisotopic_unambiguous_rna_weights
         else:
             weight_table = IUPACData.unambiguous_rna_weights
-    elif seq_type == 'protein':
+    elif seq_type == "protein":
         if monoisotopic:
             weight_table = IUPACData.monoisotopic_protein_weights
         else:
             weight_table = IUPACData.protein_weights
     else:
-        raise ValueError("Allowed seq_types are DNA, RNA or protein, not %r"
-                         % seq_type)
+        raise ValueError("Allowed seq_types are DNA, RNA or protein, not %r" % seq_type)
 
     if monoisotopic:
         water = 18.010565
@@ -439,26 +434,25 @@ def molecular_weight(seq, seq_type=None, double_stranded=False, circular=False,
         if circular:
             weight -= water
     except KeyError as e:
-        raise ValueError('%s is not a valid unambiguous letter for %s'
-                         % (e, seq_type))
-    except:
-        raise
+        raise ValueError(
+            "%s is not a valid unambiguous letter for %s" % (e, seq_type)
+        ) from None
 
-    if seq_type in ('DNA', 'RNA') and double_stranded:
+    if seq_type in ("DNA", "RNA") and double_stranded:
         seq = str(Seq(seq).complement())
         weight += sum(weight_table[x] for x in seq) - (len(seq) - 1) * water
         if circular:
             weight -= water
-    elif seq_type == 'protein' and double_stranded:
-        raise ValueError('double-stranded proteins await their discovery')
+    elif seq_type == "protein" and double_stranded:
+        raise ValueError("double-stranded proteins await their discovery")
 
     return weight
 
 
 def six_frame_translations(seq, genetic_code=1):
-    """Formatted string showing the 6 frame translations and GC content.
+    """Return pretty string showing the 6 frame translations and GC content.
 
-    nice looking 6 frame translation with GC content - code from xbbtools
+    Nice looking 6 frame translation with GC content - code from xbbtools
     similar to DNA Striders six-frame translation
 
     >>> from Bio.SeqUtils import six_frame_translations
@@ -479,70 +473,53 @@ def six_frame_translations(seq, genetic_code=1):
     <BLANKLINE>
     <BLANKLINE>
 
-    """
+    """  # noqa for pep8 W291 trailing whitespace
     from Bio.Seq import reverse_complement, translate
+
     anti = reverse_complement(seq)
     comp = anti[::-1]
     length = len(seq)
     frames = {}
     for i in range(0, 3):
         fragment_length = 3 * ((length - i) // 3)
-        frames[i + 1] = translate(seq[i:i + fragment_length], genetic_code)
-        frames[-(i + 1)] = translate(anti[i:i + fragment_length], genetic_code)[::-1]
+        frames[i + 1] = translate(seq[i : i + fragment_length], genetic_code)
+        frames[-(i + 1)] = translate(anti[i : i + fragment_length], genetic_code)[::-1]
 
     # create header
     if length > 20:
-        short = '%s ... %s' % (seq[:10], seq[-10:])
+        short = "%s ... %s" % (seq[:10], seq[-10:])
     else:
         short = seq
-    header = 'GC_Frame: '
-    for nt in ['a', 't', 'g', 'c']:
-        header += '%s:%d ' % (nt, seq.count(nt.upper()))
+    header = "GC_Frame: "
+    for nt in ["a", "t", "g", "c"]:
+        header += "%s:%d " % (nt, seq.count(nt.upper()))
 
-    header += '\nSequence: %s, %d nt, %0.2f %%GC\n\n\n' % (short.lower(), length, GC(seq))
+    header += "\nSequence: %s, %d nt, %0.2f %%GC\n\n\n" % (
+        short.lower(),
+        length,
+        GC(seq),
+    )
     res = header
 
     for i in range(0, length, 60):
-        subseq = seq[i:i + 60]
-        csubseq = comp[i:i + 60]
+        subseq = seq[i : i + 60]
+        csubseq = comp[i : i + 60]
         p = i // 3
-        res += '%d/%d\n' % (i + 1, i / 3 + 1)
-        res += '  ' + '  '.join(frames[3][p:p + 20]) + '\n'
-        res += ' ' + '  '.join(frames[2][p:p + 20]) + '\n'
-        res += '  '.join(frames[1][p:p + 20]) + '\n'
+        res += "%d/%d\n" % (i + 1, i / 3 + 1)
+        res += "  " + "  ".join(frames[3][p : p + 20]) + "\n"
+        res += " " + "  ".join(frames[2][p : p + 20]) + "\n"
+        res += "  ".join(frames[1][p : p + 20]) + "\n"
         # seq
-        res += subseq.lower() + '%5d %%\n' % int(GC(subseq))
-        res += csubseq.lower() + '\n'
+        res += subseq.lower() + "%5d %%\n" % int(GC(subseq))
+        res += csubseq.lower() + "\n"
         # - frames
-        res += '  '.join(frames[-2][p:p + 20]) + ' \n'
-        res += ' ' + '  '.join(frames[-1][p:p + 20]) + '\n'
-        res += '  ' + '  '.join(frames[-3][p:p + 20]) + '\n\n'
+        res += "  ".join(frames[-2][p : p + 20]) + " \n"
+        res += " " + "  ".join(frames[-1][p : p + 20]) + "\n"
+        res += "  " + "  ".join(frames[-3][p : p + 20]) + "\n\n"
     return res
 
-# }}}
-
-
-def _test():
-    """Run the module's doctests (PRIVATE)."""
-    import os
-    import doctest
-    if os.path.isdir(os.path.join("..", "Tests")):
-        print("Running doctests...")
-        cur_dir = os.path.abspath(os.curdir)
-        os.chdir(os.path.join("..", "Tests"))
-        doctest.testmod()
-        os.chdir(cur_dir)
-        del cur_dir
-        print("Done")
-    elif os.path.isdir(os.path.join("Tests")):
-        print("Running doctests...")
-        cur_dir = os.path.abspath(os.curdir)
-        os.chdir(os.path.join("Tests"))
-        doctest.testmod()
-        os.chdir(cur_dir)
-        del cur_dir
-        print("Done")
-
 
 if __name__ == "__main__":
-    _test()
+    from Bio._utils import run_doctest
+
+    run_doctest()
