@@ -53,7 +53,6 @@ appropriate::
             pass # do something with the motif
 """
 
-from __future__ import print_function
 
 import warnings
 from Bio import BiopythonWarning
@@ -62,8 +61,9 @@ from Bio import MissingPythonDependencyError
 try:
     import MySQLdb as mdb
 except ImportError:
-    raise MissingPythonDependencyError("Install MySQLdb if you want to use "
-                                       "Bio.motifs.jaspar.db")
+    raise MissingPythonDependencyError(
+        "Install MySQLdb if you want to use Bio.motifs.jaspar.db"
+    )
 
 from Bio.motifs import jaspar, matrix
 
@@ -71,7 +71,7 @@ from Bio.motifs import jaspar, matrix
 JASPAR_DFLT_COLLECTION = "CORE"
 
 
-class JASPAR5(object):
+class JASPAR5:
     """Class representing a JASPAR5 database.
 
     Class representing a JASPAR5 DB. The methods within are loosely based
@@ -165,10 +165,22 @@ class JASPAR5(object):
         return self.fetch_motifs(collection=None, tf_name=name)
 
     def fetch_motifs(
-        self, collection=JASPAR_DFLT_COLLECTION, tf_name=None, tf_class=None,
-        tf_family=None, matrix_id=None, tax_group=None, species=None,
-        pazar_id=None, data_type=None, medline=None, min_ic=0, min_length=0,
-        min_sites=0, all=False, all_versions=False
+        self,
+        collection=JASPAR_DFLT_COLLECTION,
+        tf_name=None,
+        tf_class=None,
+        tf_family=None,
+        matrix_id=None,
+        tax_group=None,
+        species=None,
+        pazar_id=None,
+        data_type=None,
+        medline=None,
+        min_ic=0,
+        min_length=0,
+        min_sites=0,
+        all=False,
+        all_versions=False,
     ):
         """Fetch jaspar.Record (list) of motifs using selection criteria.
 
@@ -240,7 +252,7 @@ class JASPAR5(object):
             data_type=data_type,
             medline=medline,
             all=all,
-            all_versions=all_versions
+            all_versions=all_versions,
         )
 
         record = jaspar.Record()
@@ -271,9 +283,7 @@ class JASPAR5(object):
             just use the first column sum.
             """
             if min_sites:
-                num_sites = sum(
-                    motif.counts[nt][0] for nt in motif.alphabet.letters
-                )
+                num_sites = sum(motif.counts[nt][0] for nt in motif.alphabet.letters)
                 if num_sites < min_sites:
                     continue
 
@@ -284,8 +294,11 @@ class JASPAR5(object):
     def _fetch_latest_version(self, base_id):
         """Get the latest version number for the given base_id (PRIVATE)."""
         cur = self.dbh.cursor()
-        cur.execute("""select VERSION from MATRIX where BASE_id = %s
-                       order by VERSION desc limit 1""", (base_id,))
+        cur.execute(
+            "select VERSION from MATRIX where BASE_id = %s order by VERSION"
+            " desc limit 1",
+            (base_id,),
+        )
 
         row = cur.fetchone()
 
@@ -293,10 +306,12 @@ class JASPAR5(object):
         if row:
             latest = row[0]
         else:
-            warnings.warn("Failed to fetch latest version number for JASPAR "
-                          "motif with base ID '{0}'. "
-                          "No JASPAR motif with this base ID appears to exist "
-                          "in the database.".format(base_id), BiopythonWarning)
+            warnings.warn(
+                "Failed to fetch latest version number for JASPAR motif"
+                f" with base ID '{base_id}'. No JASPAR motif with this"
+                " base ID appears to exist in the database.",
+                BiopythonWarning,
+            )
 
         return latest
 
@@ -306,8 +321,10 @@ class JASPAR5(object):
         Also checks if this combo exists or not.
         """
         cur = self.dbh.cursor()
-        cur.execute("""select id from MATRIX where BASE_id = %s
-                       and VERSION = %s""", (base_id, version))
+        cur.execute(
+            "select id from MATRIX where BASE_id = %s and VERSION = %s",
+            (base_id, version),
+        )
 
         row = cur.fetchone()
 
@@ -315,26 +332,32 @@ class JASPAR5(object):
         if row:
             int_id = row[0]
         else:
-            warnings.warn("Failed to fetch internal database ID for JASPAR "
-                          "motif with matrix ID '{0}.{1}'. "
-                          "No JASPAR motif with this matrix ID appears to "
-                          "exist.".format(base_id, version), BiopythonWarning)
+            warnings.warn(
+                "Failed to fetch internal database ID for JASPAR motif"
+                f" with matrix ID '{base_id}.{version}'. No JASPAR motif"
+                " with this matrix ID appears to exist.",
+                BiopythonWarning,
+            )
 
         return int_id
 
     def _fetch_motif_by_internal_id(self, int_id):
         """Fetch basic motif information (PRIVATE)."""
         cur = self.dbh.cursor()
-        cur.execute("""select BASE_ID, VERSION, COLLECTION, NAME from MATRIX
-                       where id = %s""", (int_id,))
+        cur.execute(
+            "select BASE_ID, VERSION, COLLECTION, NAME from MATRIX where id = %s",
+            (int_id,),
+        )
 
         row = cur.fetchone()
 
         # This should never happen as it is an internal method. If it does
         # we should probably raise an exception
         if not row:
-            warnings.warn("Could not fetch JASPAR motif with internal "
-                          "ID = {0}".format(int_id), BiopythonWarning)
+            warnings.warn(
+                f"Could not fetch JASPAR motif with internal ID = {int_id}",
+                BiopythonWarning,
+            )
             return None
 
         base_id = row[0]
@@ -348,13 +371,10 @@ class JASPAR5(object):
         counts = self._fetch_counts_matrix(int_id)
 
         # Create new JASPAR motif
-        motif = jaspar.Motif(
-            matrix_id, name, collection=collection, counts=counts
-        )
+        motif = jaspar.Motif(matrix_id, name, collection=collection, counts=counts)
 
         # fetch species
-        cur.execute("""select TAX_ID from MATRIX_SPECIES
-                       where id = %s""", (int_id,))
+        cur.execute("select TAX_ID from MATRIX_SPECIES where id = %s", (int_id,))
         tax_ids = []
         rows = cur.fetchall()
         for row in rows:
@@ -381,8 +401,7 @@ class JASPAR5(object):
         motif.acc = accs
 
         # fetch remaining annotation as tags from the ANNOTATION table
-        cur.execute("""select TAG, VAL from MATRIX_ANNOTATION
-                       where id = %s""", (int_id,))
+        cur.execute("select TAG, VAL from MATRIX_ANNOTATION where id = %s", (int_id,))
         rows = cur.fetchall()
         for row in rows:
             attr = row[0]
@@ -402,10 +421,8 @@ class JASPAR5(object):
             elif attr == "comment":
                 motif.comment = val
             else:
-                """
-                TODO If we were to implement additional abitrary tags
-                motif.tag(attr, val)
-                """
+                # TODO If we were to implement additional abitrary tags
+                # motif.tag(attr, val)
                 pass
 
         return motif
@@ -421,8 +438,10 @@ class JASPAR5(object):
         for base in "ACGT":
             base_counts = []
 
-            cur.execute("""select val from MATRIX_DATA where ID = %s
-                           and row = %s order by col""", (int_id, base))
+            cur.execute(
+                "select val from MATRIX_DATA where ID = %s and row = %s order by col",
+                (int_id, base),
+            )
 
             rows = cur.fetchall()
             for row in rows:
@@ -433,10 +452,19 @@ class JASPAR5(object):
         return matrix.GenericPositionMatrix("ACGT", counts)
 
     def _fetch_internal_id_list(
-        self, collection=JASPAR_DFLT_COLLECTION, tf_name=None, tf_class=None,
-        tf_family=None, matrix_id=None, tax_group=None, species=None,
-        pazar_id=None, data_type=None, medline=None, all=False,
-        all_versions=False
+        self,
+        collection=JASPAR_DFLT_COLLECTION,
+        tf_name=None,
+        tf_class=None,
+        tf_family=None,
+        matrix_id=None,
+        tax_group=None,
+        species=None,
+        pazar_id=None,
+        data_type=None,
+        medline=None,
+        all=False,
+        all_versions=False,
     ):
         """Fetch list of internal JASPAR motif IDs.
 
@@ -500,9 +528,7 @@ class JASPAR5(object):
                 for id in matrix_id:
                     # ignore vesion here, this is a stupidity filter
                     (base_id, version) = jaspar.split_jaspar_id(id)
-                    cur.execute(
-                        "select ID from MATRIX where BASE_ID = %s", (base_id,)
-                    )
+                    cur.execute("select ID from MATRIX where BASE_ID = %s", (base_id,))
 
                     rows = cur.fetchall()
                     for row in rows:
@@ -718,8 +744,9 @@ class JASPAR5(object):
                     int_ids.append(id)
 
         if len(int_ids) < 1:
-            warnings.warn("Zero motifs returned with current select critera",
-                          BiopythonWarning)
+            warnings.warn(
+                "Zero motifs returned with current select critera", BiopythonWarning
+            )
 
         return int_ids
 
@@ -731,10 +758,12 @@ class JASPAR5(object):
         """
         cur = self.dbh.cursor()
 
-        cur.execute("select count(*) from MATRIX where "
-                    "BASE_ID = (select BASE_ID from MATRIX where ID = %s) "
-                    "and VERSION > (select VERSION from MATRIX where ID = %s)",
-                    (int_id, int_id))
+        cur.execute(
+            "select count(*) from MATRIX where "
+            "BASE_ID = (select BASE_ID from MATRIX where ID = %s) "
+            "and VERSION > (select VERSION from MATRIX where ID = %s)",
+            (int_id, int_id),
+        )
 
         row = cur.fetchone()
 
